/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.util.monolog.wrapper.common;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.MonologFactory;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class AbstractFactory
	implements MonologFactory, Configurable {

    /**
     * Inidicates if the monolog wrapper must be logged itself.
     */
    public static boolean debug = false;

    static {
        debug = new Boolean(System.getProperty("monolog.debug")).booleanValue();
    }

    /**
     * This method must be only used to debug the Monolog wrappers.
     * To active the log of monolog assign the "true" value to the system
     * property "monolog.debug".
     *
     * @param m the message to log.
     */
    public static void debug(String m) {
        if (debug) {
            System.out.println(m);
        }
    }

    /**
     * The default resource bundle of this factory
     */
    protected String resourceBundleName = null;

    /**
     * This field references the level instances by their names.<br/>
     * key = a level name<br/>
     * value = the unique Level instance linked to the name.
     */
    protected Map nameToLevel = null;

    /**
     * This field reference the level names by their integer value.<br/>
     * key = a java.lang.Integer which the value is the level<br/>
     * value = a String or an ArrayList of String. The strings represent the
     * name which match to the integer value. Indeed both name can be associated
     * to the same integer value.
     */
    protected Map intToNames = null;

    /**
     * This field references the handler instance by their names.<br/>
     * key = a String object which is an handler name.
     * value = the unique handler instance which has the key for name.
     */
    protected Map handlers = null;

    /**
     * It initializes the default monolog level: INHERIT, DEBUG, INFO, WARN,
     * ERROR, FATAL
     */
    public AbstractFactory() {
        intToNames = new HashMap();
        nameToLevel = new HashMap();
        handlers = new HashMap();
        defineLevel(BasicLevel.LEVEL_INHERIT);
        defineLevel(BasicLevel.LEVEL_DEBUG);
        defineLevel(BasicLevel.LEVEL_INFO);
        defineLevel(BasicLevel.LEVEL_WARN);
        defineLevel(BasicLevel.LEVEL_ERROR);
        defineLevel(BasicLevel.LEVEL_FATAL);
    }

    /**
     * Insert a level into the data structure.<br/>
     * If the level name is already used with other integer value, the null
     * value is returned.<br/>
     * If the level name is already used with the same integer value, the level
     * found in the data structure is returned.<br/>
     *
     * @param l the Level instance which must be inserted.
     * @return the Level instance or a null value.
     */
    private Level defineLevel(Level l) {
        //System.out.println("def(" + l + ") begin");
        String name = l.getName();
        int value = l.getIntValue();
        Level res = (Level) nameToLevel.get(name);
        if (res != null) {
            // The name is already defined.
            return (res.getIntValue() == value ? res : null);
        }
        else {
            res = l;
            nameToLevel.put(name, res);
            Integer i = new Integer(value);
            Object temp = intToNames.get(i);
            if (temp != null) {
                if (temp instanceof String) {
                    if (!((String) temp).equalsIgnoreCase(name)) {
                        // The int value has already another name
                        // Add the new name to the other
                        ArrayList al = new ArrayList(5);
                        al.add(temp);
                        al.add(name);
                        intToNames.put(i, al);
                    }
                }
                else if (temp instanceof ArrayList) {
                    // The int value has already several another name
                    ArrayList al = (ArrayList) temp;
                    if (!al.contains(name)) {
                        // Add the new name to the others
                        al.add(name);
                    }
                }
            }
            else {
                // The int value does not have any name
                intToNames.put(i, name);
            }
        }
        //System.out.println("def(" + l + ") end");
        return res;
    }

    // IMPLEMENTATION OF THE Configurable INTERFACE //
    //----------------------------------------------//

    public abstract void configure(Properties prop) throws Exception;

	// IMPLEMENTATION OF INTERFACE LoggerFactory //
	//-------------------------------------------//

	public abstract Logger getLogger(String key);

    public abstract Logger[] getLoggers();

	public String getResourceBundleName() {
		return resourceBundleName;
	}

	public void setResourceBundleName(String rbn) {
		resourceBundleName = rbn;
	}


	// IMPLEMENTATION OF THE HandlerFactory INTERFACE //
	//------------------------------------------------//

	public abstract Handler createHandler(String hn, byte handlertype);

	public Handler[] getHandlers() {
		return (Handler[]) handlers.values().toArray(new Handler[0]);
	}

	public Handler getHandler(String hn) {
		return (Handler) handlers.get(hn);
	}

	public Handler removeHandler(String hn) {
		return (Handler) handlers.remove(hn);
	}

	// IMPLEMENTATION OF THE LevelFactory INTERFACE //
	//-----------------------------------------------//

	public Level defineLevel(String name, int value) {
		return defineLevel(new LevelImpl(name, value));
	}

	public Level defineLevel(String name, String value) {
		return defineLevel(new LevelImpl(name, value, this));
	}

	public Level getLevel(String name) {
		return (Level) nameToLevel.get(name);
	}

	public Level getLevel(int value) {
		Object temp = intToNames.get(new Integer(value));
		if (temp == null) {
			return null;
		}
		else if (temp instanceof String) {
			return getLevel((String) temp);
		}
		else if (temp instanceof ArrayList) {
			return getLevel((String) ((ArrayList) temp).get(0));
		}
		return null;
	}

	public Level[] getLevels() {
		return (Level[]) nameToLevel.values().toArray(new Level[0]);
	}

	public void removeLevel(String name) {
		Level removed = (Level) nameToLevel.remove(name);
		if (removed != null) {
			Integer i = new Integer(removed.getIntValue());
			Object temp = intToNames.get(i);
			if (temp instanceof String) {
				intToNames.remove(i);
			}
			else if (temp instanceof ArrayList) {
				((ArrayList) temp).remove(name);
			}
		}
	}
}