/**
 * Copyright (C) 2002
 */

package org.objectweb.util.monolog;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.textui.TestRunner;

import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.api.HandlerFactory;

/**
 * This test verifies an implementation of the Handler interface and an
 * implementation of the HandlerFactory interface.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class TestHandler extends TestCase {

	/**
	 * This constant field contains the list of the setter name needed to
	 * initialize this class.
	 */
	public static final String[] SETTER_METHODS = {"setHandlerFactoryClassName"};

	protected HandlerFactory hf = null;

	private Handler hc = null;

	public TestHandler() {
		super("");
	}

	/**
	 * It assigns the HandlerFactory class name. This method tries to get an
	 * instance with this class name.
	 */
	public void setHandlerFactoryClassName(String lfcn) {
		try {
			hf = (HandlerFactory) Class.forName(lfcn).newInstance();
		}
		catch (ClassCastException e) {
			throw new UnsupportedOperationException(
				"The specified class is not a Handler factory: " + lfcn);
		}
		catch (Exception e) {
			throw new UnsupportedOperationException(
				"Handler factory class is not availlable: " + lfcn);
		}
	}

	/**
	 * For running the TestHandler suite standalone. A particular TestSuite is
	 * used to initialized the class instance by setter methods.
	 * One parameter is required:
	 * <ul>
	 * <li>a HandlerFactory class name</li>
	 * </ul>
	 */
	public static void main(String args[]) {
		if (args.length < 1) {
			System.out.println("Syntax error !");
			System.out.println("java TestHandler <handler factory class name>");
			System.exit(12);
		}
		Object[] params = {args[0]};
		try {
			TestSuite suite =
				new TestSuite(TestHandler.class, SETTER_METHODS, params);
			TestRunner.run(suite);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static TestSuite getTestSuite(String lfcn) {
		Object[] params = {lfcn};
		try {
			return new TestSuite(TestHandler.class, SETTER_METHODS, params);
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	// ------ TEST METHODS ------ //
	//----------------------------//

	/**
	 * It tests to define and configure a console handler.
	 * see testHandlerDef(byte) method
	 */
	public void testConsoleHandlerDef() {
		testHandlerDef(Handler.CONSOLE_HANDLER_TYPE);
	}

	/**
	 * It tests to define and configure a file handler.
	 * see testHandlerDef(byte) method
	 */
	public void testFileHandlerDef() {
		testHandlerDef(Handler.FILE_HANDLER_TYPE);
	}

	/**
	 * It tests to define and configure a rolling file handler.
	 * see testHandlerDef(byte) method
	 */
	public void testRollingFileHandlerDef() {
		testHandlerDef(Handler.ROLLING_FILE_HANDLER_TYPE);
	}

	/**
	 * It creates one handler. Then it fetches it by its name with the
	 * getHandler(String) method. It checks the name, properties, the
	 * number of handler in the HandlerFactory.
	 */
	protected void testHandlerDef(byte handlerType) {
		hc = hf.createHandler("MyHandler", handlerType);
		hc.setAttribute("toto_key", "toto_value");
		hc.setAttribute("titi.key", "titi.value");

		assertEquals("assigns property",
			"toto_value", hc.getAttribute("toto_key"));
		assertEquals("assigns property with dot",
			"titi.value", hc.getAttribute("titi.key"));
		assertEquals("assigns a handler name",
			"MyHandler", hc.getName());


		assertEquals("Handler not correctly registered into MonologConf",
			hc, hf.getHandler("MyHandler"));
		assertEquals("Handler not correctly registered into MonologConf (time 2)",
			hc, hf.getHandler("MyHandler"));
		hc = hf.getHandler("MyHandler");

		assertEquals("assigns property after getHandlerConf(String)",
			"toto_value", hc.getAttribute("toto_key"));
		assertEquals("assigns property with dot after getHandlerConf(String)",
			"titi.value", hc.getAttribute("titi.key"));
		assertEquals("assigns a handler name after getHandlerConf(String)",
			"MyHandler", hc.getName());

		assertEquals("getHandler() with one HandlerConf",
			1, hf.getHandlers().length);

		hf.removeHandler("MyHandler");

		assertEquals("removeHandler with one HandlerConf",
			0, hf.getHandlers().length);
	}

	/**
	 * It tests to fetch several console handlers by their name.
	 * see the testgetHandlerByName(byte) method
	 */
	public void testgetConsoleHandlerByName() {
		testgetHandlerByName(Handler.CONSOLE_HANDLER_TYPE);
	}

	/**
	 * It tests to fetch several file handlers by their name.
	 * see testgetHandlerByName(byte) method
	 */
	public void testgetFileHandlerByName() {
		testgetHandlerByName(Handler.FILE_HANDLER_TYPE);
	}

	/**
	 * It tests to fetch several rolling file handlers by their name.
	 * see testgetHandlerByName(byte) method
	 */
	public void testgetRollingFileHandlerByName() {
		testgetHandlerByName(Handler.ROLLING_FILE_HANDLER_TYPE);
	}

	/**
	 * It creates 20 handlers. Then it fetches them by their name with the
	 * getHandler(String) method.
	 */
	protected void testgetHandlerByName(byte handlerType) {
		int iternumber = 20;
		for (int i = 0; i < iternumber; i++) {
			hc = hf.createHandler("MyHandler" + i, handlerType);
			hc.setAttribute("toto_key" + i, "toto_value" + i);
			hc.setAttribute("titi.key" + i, "titi.value" + i);
		}
		for (int i = 0; i < iternumber; i++) {
			String name = "MyHandler" + i;
			hc = hf.getHandler(name);
			assertNotNull("fetch " + name, hc);
			assertEquals("name value by fetch " + name, name, hc.getName());

			assertEquals("assigns property by fetch " + name,
				"toto_value" + i, hc.getAttribute("toto_key" + i));
			assertEquals("assigns property with dot by fetch " + name,
				"titi.value" + i, hc.getAttribute("titi.key" + i));
		}
	}

	/**
	 * It tests to fetch several console handlers by the getHandler() method.
	 * see testGetAllHandler(byte) method
	 */
	public void testGetAllHandlerConsole() {
		testGetAllHandler(Handler.CONSOLE_HANDLER_TYPE);
	}

	/**
	 * It tests to fetch several file handlers by the getHandler() method.
	 * see testGetAllHandler(byte) method
	 */
	public void testGetAllHandlerFile() {
		testGetAllHandler(Handler.FILE_HANDLER_TYPE);
	}

	/**
	 * It tests to fetch several rolling file handlers by the getHandler()
	 * method.
	 * see testGetAllHandler(byte) method
	 */
	public void testGetAllHandlerRollingFile() {
		testGetAllHandler(Handler.ROLLING_FILE_HANDLER_TYPE);
	}

	/**
	 * It creates 20 handlers. Then it fetches them by the
	 * getHandler() method.
	 */
	protected void testGetAllHandler(byte handlerType) {
		int iternumber = 20;
		for (int i = 0; i < iternumber; i++) {
			hc = hf.createHandler("MyHandler" + i, handlerType);
			hc.setAttribute("toto_key" + i, "toto_value" + i);
			hc.setAttribute("titi.key" + i, "titi.value" + i);
		}
		Handler[] hcs = hf.getHandlers();
		assertEquals("getHandler with " + iternumber + " HandlerConf",
			iternumber, hcs.length);

		Handler[] hcs2 = new Handler[hcs.length];
		for (int i = 0; i < hcs.length; i++) {
			hc = hcs[i];
			int j = Integer.parseInt(
				hc.getName().substring(9,hc.getName().length()));
			assertNull("Duplicate HandlerConf", hcs2[j]);
			hcs2[j] = hc;
			String name = "MyHandler" + j;
			assertNotNull("fetch " + name, hc);
			assertEquals("name value by fetch " + name, name, hc.getName());

			assertEquals("assigns property by fetch " + name,
				"toto_value" + j, hc.getAttribute("toto_key" + j));
			assertEquals("assigns property with dot by fetch " + name,
				"titi.value" + j, hc.getAttribute("titi.key" + j));
		}
	}

	/**
	 * It tests to remove several console handlers.
	 * see testRemoveHandler(byte) method
	 */
	public void testRemoveConsoleHandler() {
		testRemoveHandler(Handler.CONSOLE_HANDLER_TYPE);
	}

	/**
	 * It tests to remove several file handlers.
	 * see testRemoveHandler(byte) method
	 */
	public void testRemoveFileHandler() {
		testRemoveHandler(Handler.FILE_HANDLER_TYPE);
	}

	/**
	 * It tests to remove several rolling file handlers.
	 * see testRemoveHandler(byte) method
	 */
	public void testRemoveRollingFileHandler() {
		testRemoveHandler(Handler.ROLLING_FILE_HANDLER_TYPE);
	}

	/**
	 * It creates 20 handlers. Then it removes them.
	 */
	public void testRemoveHandler(byte handlerType) {
		int iternumber = 20;
		for (int i = 0; i < iternumber; i++) {
			hc = hf.createHandler("MyHandler" + i, handlerType);
			hc.setAttribute("toto_key" + i, "toto_value" + i);
			hc.setAttribute("titi.key" + i, "titi.value" + i);
		}

		Handler[] hcs = hf.getHandlers();
		for (int i = 0; i < hcs.length; i++) {
			String name = "MyHandler" + i;
			hf.removeHandler(name);
			Assert.assertNull("Remove " + name, hf.getHandler(name));
		}
	}

	/**
	 * It creates 20 handlers. Then it fetches them by their name with the
	 * getHandler(String) method.
	 */
	protected void testgetHandlerByName() {
		int iternumber = 20;
		for (int i = 0; i < iternumber; i++) {
			if (i<(iternumber/3)) {
				hc = hf.createHandler("MyHandler" + i,
					Handler.CONSOLE_HANDLER_TYPE);
			}
			else if (i<((2*iternumber)/3)) {
				hc = hf.createHandler("MyHandler" + i,
					Handler.FILE_HANDLER_TYPE);
			}
			else {
				hc = hf.createHandler("MyHandler" + i,
					Handler.ROLLING_FILE_HANDLER_TYPE);
			}
			hc.setAttribute("toto_key" + i, "toto_value" + i);
			hc.setAttribute("titi.key" + i, "titi.value" + i);
		}
		for (int i = 0; i < iternumber; i++) {
			String name = "MyHandler" + i;
			hc = hf.getHandler(name);
			assertNotNull("fetch " + name, hc);
			assertEquals("name value by fetch " + name,
				name, hc.getName());

			assertEquals("assigns property by fetch " + name,
				"toto_value" + i, hc.getAttribute("toto_key" + i));
			assertEquals("assigns property with dot by fetch " + name,
				"titi.value" + i, hc.getAttribute("titi.key" + i));
		}
	}
}
