/**
 * Copyright (C) 2002
 */

package org.objectweb.util.monolog;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.api.Level;
import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 * It verifies a Level implementation and a LevelFactory implementation.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class TestLevel extends TestCase {

	/**
	 * This constant field contains the list of the setter name needed to
	 * initialize this class.
	 */
	public static final String[] SETTER_METHODS = {"setLevelFactoryClassName"};

	protected LevelFactory lf = null;

	private Level l = null;

	public TestLevel() {
		super("");
	}

	/**
	 * It assigns the LevelFactory class name. This method tries to get an
	 * instance with this class name.
	 */
	public void setLevelFactoryClassName(String lfcn) {
		try {
			lf = (LevelFactory) Class.forName(lfcn).newInstance();
		}
		catch (ClassCastException e) {
			throw new UnsupportedOperationException(
				"The specified class is not a Level factory: " + lfcn);
		}
		catch (Exception e) {
			throw new UnsupportedOperationException(
				"Level factory class is not availlable: " + lfcn);
		}
	}

	/**
	 * For running the TestLevel suite standalone. A particular TestSuite is
	 * used to initialized the class instance by setter methods.
	 * One parameter is required:
	 * <ul>
	 * <li>a LevelFactory class name</li>
	 * </ul>
	 */
	public static void main(String args[]) {
		if (args.length < 1) {
			System.out.println("Syntax error !");
			System.out.println("java TestLevel <level factory class name>");
			System.exit(12);
		}
		Object[] params = {args[0]};
		try {
			TestSuite suite =
				new TestSuite(TestLevel.class, SETTER_METHODS, params);
			TestRunner.run(suite);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static TestSuite getTestSuite(String lfcn) {
		Object[] params = {lfcn};
		try {
			return new TestSuite(TestLevel.class, SETTER_METHODS, params);
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	// ------ TEST METHODS ------ //
	//----------------------------//

	/**
	 * This method tests if it is possible to define a LevelConf with a
	 * predefined level and the + operator: DEBUG + 1
	 */
	public void testInterLevelDef() {
		assertEquals("Predefined level unreconized",
			BasicLevel.DEBUG,
			lf.getLevel("DEBUG").getIntValue());
		l = lf.defineLevel("MY_LEVEL", "DEBUG + 1");
		assertNotNull("Null level", l);
		assertEquals("intermediate level definition",
			BasicLevel.DEBUG + 1,
			l.getIntValue());
	}

	/**
	 * This method tests if it is possible to define a LevelConf with an
	 * other level and the + operator:
	 * <ul>
	 * <li>MY_LEVEL = DEBUG + 1</li>
	 * <li>MY_LEVEL2 = MY_LEVEL + 1</li>
	 * </ul>
	 */
	public void testInterLevelDefWithOther() {
		lf.defineLevel("MY_LEVEL", "DEBUG + 1");
		l = lf.defineLevel("MY_LEVEL2", "MY_LEVEL + 1");
		assertEquals(
			"intermediate level definition based on another intermediate level",
			BasicLevel.DEBUG + 2,
			l.getIntValue());
	}

	/**
	 * This method tests if it is possible to define a LevelConf with a
	 * predefined level and the minus operator: DEBUG + 1
	 */
	public void testInterLevelDefByMinus() {
		l = lf.defineLevel("TOTO", "DEBUG - 1");
		assertEquals(
			"intermediate level definition minus operator",
			BasicLevel.DEBUG - 1,
			l.getIntValue());
	}

	/**
	 * This method tests if it is possible to define a LevelConf with a
	 * predefined level, the + operator and a number with several figures:
	 * DEBUG + 156
	 */
	public void testInterLevelDefWithFigures() {
		l = lf.defineLevel("TOTO2", "INFO +156 ");
		assertEquals(
			"intermediate level definition, number with several figures",
			BasicLevel.INFO + 156,
			l.getIntValue());
	}

	/**
	 * This method tests if it is possible to define a LevelConf with a
	 * predefined level witten with a combinaison of lower case and upper case
	 * and the + operator: InFO + 1
	 */
	public void testInterLevelWithLowerCase() {
		l = lf.defineLevel("TOTO3", "   InFO + 1 ");
		assertEquals(
			"intermediate level definition, level name with lower case",
			BasicLevel.INFO + 1,
			l.getIntValue());
	}

	/**
	 * This method tests if it is possible to define a LevelConf with just a
	 * number.
	 */
	public void testInterLevelJustNb() {
		l = lf.defineLevel("TOTO4", "23455");
		assertEquals(
			"intermediate level definition, level value is just a number",
			23455,
			l.getIntValue());
		l = lf.defineLevel("TOTO7", 23455);
		assertEquals(
			"intermediate level definition, level value is just a number",
			23455,
			l.getIntValue());

	}

	/**
	 * This method tests if it is possible to define a LevelConf with just an
	 * other level.
	 */
	public void testInterLevelJustOther() {
		lf.defineLevel("TOTO3", "   InFO + 1 ");
		l = lf.defineLevel("TOTO5", "TOTO3");
		assertEquals(
			"intermediate level definition, level value is just another intermediate level",
			BasicLevel.INFO + 1,
			l.getIntValue());
	}

}
